System.register(["tslib", "./http/JsonError"], function (_export, _context) {
  "use strict";

  var __awaiter, JsonError, FetchBuilder;
  function buildUrl(strings, ...p) {
    let r = "";
    for (let index = 0; index < strings.length; index++) {
      const element = strings[index];
      r += element;
      if (index < p.length) {
        r += encodeURIComponent(p[index]);
      }
    }
    return r;
  }
  _export({
    buildUrl: buildUrl,
    default: void 0
  });
  return {
    setters: [function (_tslib) {
      __awaiter = _tslib.__awaiter;
    }, function (_httpJsonError) {
      JsonError = _httpJsonError.default;
    }],
    execute: function () {
      _export("default", FetchBuilder = class FetchBuilder {
        static get(url) {
          return this.method(url, "GET");
        }
        static put(url) {
          return this.method(url, "PUT");
        }
        static post(url) {
          return this.method(url, "POST");
        }
        static delete(url) {
          return this.method(url, "DELETE");
        }
        static method(url, method) {
          return new FetchBuilder({
            url,
            method
          });
        }
        constructor(request) {
          this.request = request;
        }
        form(name, value) {
          var _a;
          const body = (_a = this.request.body) !== null && _a !== void 0 ? _a : new FormData();
          body.append(name, value);
          return this.append({
            body
          });
        }
        jsonBody(body, encode = true) {
          var _a;
          if (encode) {
            body = JSON.stringify(body);
          }
          const headers = (_a = this.request.headers) !== null && _a !== void 0 ? _a : {};
          headers["content-type"] = "application/json";
          return this.append({
            body,
            headers
          });
        }
        header(name, value) {
          var _a;
          const headers = (_a = this.request.headers) !== null && _a !== void 0 ? _a : {};
          headers[name] = value;
          return this.append({
            headers
          });
        }
        path(name, value, encode = true) {
          let url = this.request.url;
          if (encode) {
            value = encodeURIComponent(value);
          }
          url = url.replace(name, value);
          return this.append({
            url
          });
        }
        query(name, value, encode = true) {
          let url = this.request.url;
          if (encode) {
            value = encodeURIComponent(value);
          }
          name = encodeURIComponent(name);
          if (url.indexOf("?") === -1) {
            url += `?${name}=${value}`;
          } else {
            url += `&${name}=${value}`;
          }
          return this.append({
            url
          });
        }
        responseAsText(ensureSuccess = true) {
          return __awaiter(this, void 0, void 0, function* () {
            const r = yield this.response();
            if (ensureSuccess) {
              yield this.ensureSuccess(r);
            }
            return yield r.text();
          });
        }
        responseAsBlob(ensureSuccess = true) {
          return __awaiter(this, void 0, void 0, function* () {
            const r = yield this.response();
            if (ensureSuccess) {
              yield this.ensureSuccess(r);
            }
            return yield r.blob();
          });
        }
        responseAsJson(ensureSuccess = true) {
          return __awaiter(this, void 0, void 0, function* () {
            const r = yield this.response();
            if (ensureSuccess) {
              yield this.ensureSuccess(r);
            }
            return yield r.json();
          });
        }
        response() {
          return fetch(this.request.url, this.request);
        }
        ensureSuccess(r) {
          return __awaiter(this, void 0, void 0, function* () {
            if (r.status <= 300) {
              return;
            }
            const type = r.headers.get("content-type");
            if (/\/json/i.test(type)) {
              const json = yield r.json();
              throw new JsonError(`Failed for ${this.request.url}`, json);
            }
            const text = yield r.text();
            throw new Error(`Failed for ${this.request.url}\n${text}`);
          });
        }
        append(r) {
          return new FetchBuilder(Object.assign(Object.assign({}, this.request), r));
        }
      });
    }
  };
});
//# sourceMappingURL=FetchBuilder.js.map