System.register(["./cloneSource"], function (_export, _context) {
  "use strict";

  var cloneSource, Cloner, PropertyCloner, getKey;
  _export({
    Cloner: void 0,
    PropertyCloner: void 0
  });
  return {
    setters: [function (_cloneSource) {
      cloneSource = _cloneSource.default;
    }],
    execute: function () {
      getKey = (target, value) => {
        for (const key in target) {
          if (Object.prototype.hasOwnProperty.call(target, key)) {
            const element = target[key];
            if (element === value) {
              return key;
            }
          }
        }
      };
      _export("Cloner", Cloner = class Cloner {
        static copyProperties(src, dest = {}) {
          dest[cloneSource] = src;
          for (const key in src) {
            if (Object.prototype.hasOwnProperty.call(src, key)) {
              const element = src[key];
              switch (typeof element) {
                case "bigint":
                case "boolean":
                case "number":
                case "string":
                  dest[key] = element;
                  break;
                case "object":
                  if (element === null) {
                    dest[key] = null;
                  }
                  if (element instanceof Date) {
                    dest[key] = element;
                  }
                  break;
              }
            }
          }
          return dest;
        }
        get copy() {
          const copy = Cloner.copyProperties(this.item);
          if (this.path) {
            for (const iterator of this.path) {
              this.clone(this.item, copy, iterator);
            }
          }
          return copy;
        }
        constructor(item, path) {
          this.item = item;
          this.path = path;
          this.$type = item.$type;
        }
        include(property) {
          const last = [property];
          return new PropertyCloner(this.item, this.path ? [...this.path, last] : [last], last);
        }
        clone(src, dest, paths) {
          var _a, _b, _c;
          if (!paths.length) {
            return dest;
          }
          const [first, ...others] = paths;
          let p = first(src);
          if (!p) {
            return dest;
          }
          const original = p;
          const name = getKey(src, p);
          if (Array.isArray(p)) {
            const existing = (_a = dest[name]) !== null && _a !== void 0 ? _a : dest[name] = [];
            for (let index = 0; index < p.length; index++) {
              const element = p[index];
              const existingItem = (_b = existing[index]) !== null && _b !== void 0 ? _b : existing[index] = {};
              Cloner.copyProperties(element, existingItem);
              this.clone(element, existingItem, others);
            }
            return existing;
          }
          p = Cloner.copyProperties(p, (_c = dest[name]) !== null && _c !== void 0 ? _c : undefined);
          dest[name] = p;
          this.clone(original, p, others);
          return dest;
        }
      });
      _export("PropertyCloner", PropertyCloner = class PropertyCloner extends Cloner {
        constructor(item, path, propertyPath) {
          super(item, path);
          this.propertyPath = propertyPath;
        }
        thenInclude(property) {
          this.propertyPath.push(property);
          return new PropertyCloner(this.item, this.path, this.propertyPath);
        }
      });
    }
  };
});
//# sourceMappingURL=Cloner.js.map