System.register(["tslib", "./BaseEntityService"], function (_export, _context) {
  "use strict";

  var __awaiter, BaseEntityService, EntitySet, MockEntityRestService, MockBaseEntityService, replacer, convertToSafe;
  function sleep(n) {
    return new Promise((resolve, reject) => {
      setTimeout(resolve, n);
    });
  }
  _export({
    EntitySet: void 0,
    default: void 0
  });
  return {
    setters: [function (_tslib) {
      __awaiter = _tslib.__awaiter;
    }, function (_BaseEntityService) {
      BaseEntityService = _BaseEntityService.default;
    }],
    execute: function () {
      replacer = /(\.[a-z0-9\_])|([a-z0-9\_]\())/ig;
      convertToSafe = text => {
        return text.replace(replacer, x => {
          if (x.endsWith("(")) {
            return x[0] + "?.(";
          }
          return "?" + x;
        });
      };
      _export("EntitySet", EntitySet = class EntitySet {
        get all() {
          const a = [];
          for (const key in this.primary) {
            if (Object.prototype.hasOwnProperty.call(this.primary, key)) {
              const element = this.primary[key];
              a.push(element);
            }
          }
          return a;
        }
        constructor(model, entityService) {
          this.entityService = entityService;
          this.primary = {};
          this.name = model.name;
        }
        add(...items) {
          for (const item of items) {
            this.save(item);
          }
        }
        save(item) {
          const model = this.entityService.entityContext.for(this.name);
          const keys = this.keys(item);
          const existing = this.find(keys);
          if (existing) {
            for (const key in item) {
              if (Object.prototype.hasOwnProperty.call(item, key)) {
                const element = item[key];
                existing[key] = element;
              }
            }
            return;
          }
          for (const property of model.properties) {
            if (!item.hasOwnProperty(property.name)) {
              item[property.name] = null;
            }
          }
          this.primary[JSON.stringify(keys)] = item;
        }
        delete(item) {
          const model = this.entityService.entityContext.for(this.name);
          const keys = this.keys(item);
          delete this.primary[JSON.stringify(keys)];
        }
        keys(item) {
          const model = this.entityService.entityContext.for(this.name);
          const keys = [];
          for (const iterator of model.keys) {
            keys.push(item[iterator.name]);
          }
          return keys;
        }
        find(keys) {
          const key = JSON.stringify(keys);
          return this.primary[key];
        }
      });
      MockEntityRestService = class MockEntityRestService {
        constructor(es) {
          this.es = es;
        }
        query(entity, methods, ct) {
          return __awaiter(this, void 0, void 0, function* () {
            yield sleep(200);
            if (ct.cancelled) {
              throw new Error("cancelled");
            }
            const model = this.es.entityContext.for(entity);
            let all = this.es.set({
              name: entity
            }).all;
            for (const iterator of methods.methods) {
              for (const key in iterator) {
                if (Object.prototype.hasOwnProperty.call(iterator, key)) {
                  const [expression, ...pl] = iterator[key];
                  let fx = convertToSafe(expression);
                  for (let index = 0; index < pl.length; index++) {
                    fx = fx.replaceAll(`@${index}`, `p[${index}]`);
                  }
                  fx = eval(`(p) => ${fx}`)(pl);
                  all = all[key].apply(all, fx);
                }
              }
            }
            const total = all.length;
            if (methods.start) {
              while (methods.start > 0) {
                all = all.shift();
              }
            }
            if (methods.size) {
              all.length = methods.size;
            }
            return Promise.resolve({
              items: all,
              total
            });
          });
        }
        insert(body) {
          return __awaiter(this, void 0, void 0, function* () {
            yield sleep(100);
            this.es.set({
              name: body.$type
            }).add(body);
            return Promise.resolve(body);
          });
        }
        save(body) {
          return __awaiter(this, void 0, void 0, function* () {
            yield sleep(100);
            this.es.set({
              name: body.$type
            }).add(body);
            return Promise.resolve(body);
          });
        }
        delete(body) {
          return __awaiter(this, void 0, void 0, function* () {
            yield sleep(100);
            this.es.set({
              name: body.$type
            }).delete(body);
            return Promise.resolve();
          });
        }
        bulkSave(body) {
          return __awaiter(this, void 0, void 0, function* () {
            yield sleep(100);
            for (const entity of body.keys) {
              const model = this.es.set({
                name: entity.$type
              });
              const keys = model.keys(entity);
              const existing = model.find(keys);
              if (existing) {
                for (const key in body.update) {
                  if (Object.prototype.hasOwnProperty.call(body.update, key)) {
                    const element = body.update[key];
                    existing[key] = element;
                  }
                }
              }
            }
            return Promise.resolve();
          });
        }
        bulkDelete(body) {
          return __awaiter(this, void 0, void 0, function* () {
            yield sleep(100);
            for (const entity of body.keys) {
              const model = this.es.set({
                name: entity.$type
              });
              model.delete(entity);
            }
            return Promise.resolve();
          });
        }
      };
      _export("default", MockBaseEntityService = class MockBaseEntityService extends BaseEntityService {
        constructor() {
          super(...arguments);
          this.local = {};
        }
        seed() {
          console.warn("Please override seed method for data");
        }
        set(m) {
          var _a;
          var _b, _c;
          return (_a = (_b = this.local)[_c = m.name]) !== null && _a !== void 0 ? _a : _b[_c] = new EntitySet(m, this);
        }
      });
    }
  };
});
//# sourceMappingURL=MockBaseEntityService.js.map