System.register(["tslib", "./StringHelper"], function (_export, _context) {
  "use strict";

  var __awaiter, StringHelper, QueryComposer, Query, replacer, convertToLinq;
  function append(original, ...item) {
    if (original) {
      return [...original, ...item];
    }
    return [...item];
  }
  _export({
    append: append,
    QueryComposer: void 0,
    default: void 0
  });
  return {
    setters: [function (_tslib) {
      __awaiter = _tslib.__awaiter;
    }, function (_StringHelper) {
      StringHelper = _StringHelper.default;
    }],
    execute: function () {
      replacer = /(===)|(!==)|(\(\s*\{)|(\.[a-zA-Z0-9]+)|([a-zA-Z0-9]+\s*\:\s*\{?)/g;
      _export("convertToLinq", convertToLinq = x => {
        x = x.replace(/(\s+)|((CastAs|EF)\_[0-9]\.default\.)/g, (s, first, second) => {
          if (s === first) {
            return " ";
          }
          if (s.startsWith("EF")) {
            return "EF.";
          }
          if (s.startsWith("CastAs")) {
            return "CastAs.";
          }
          return s;
        });
        replacer.lastIndex = 0;
        x = x.replace(replacer, s => {
          switch (s) {
            case "===":
              return "==";
            case "!==":
              return "!=";
            case "({":
              return "( new {";
            case ".some":
              return ".Any";
            case ".map":
              return ".Select";
            case ".filter":
              return ".Where";
            case ".find":
              return ".FirstOrDefault";
            case ".includes":
              return ".Contains";
          }
          if (s.endsWith("{")) {
            const index = s.indexOf(":");
            const v = s.substring(0, index) + " = new {";
            return v;
          }
          const trimmed = s.trim();
          if (trimmed.endsWith(":")) {
            return trimmed.substring(0, trimmed.length - 1) + " = ";
          }
          if (s.startsWith(".") && s.length > 2) {
            return s[0] + s[1].toUpperCase() + s.substring(2);
          }
          return s;
        });
        return x;
      });
      _export("QueryComposer", QueryComposer = class QueryComposer {
        constructor(separator = " || ") {
          this.separator = separator;
          this.query = "p => x => ";
          this.parameters = {};
          this.index = 0;
        }
        add(p, q) {
          if (this.index) {
            this.query += this.separator;
          }
          const suffix = `_${this.index++}`;
          let text = q.toString();
          const pfn = StringHelper.findParameter(text).trim();
          text = text.substring(text.indexOf("=>") + 2);
          const xfn = StringHelper.findParameter(text).trim();
          const target = this.parameters;
          for (const key in p) {
            if (Object.prototype.hasOwnProperty.call(p, key)) {
              const element = p[key];
              const name = `${key}${suffix}`;
              target[name] = element;
              text = text.split(`${pfn}.${key}`).join(`p.${name}`);
              text = text.split(`${xfn}.`).join(`x.`);
            }
          }
          text = text.substring(text.indexOf("=>") + 2);
          this.query += text;
        }
        asQuery() {
          return [this.parameters, this.query];
        }
      });
      _export("default", Query = class Query {
        constructor(context, methods = []) {
          this.context = context;
          this.methods = methods;
          if (!methods) {
            throw new Error("Methods cannot be empty");
          }
        }
        where(tOrP, q) {
          return this.process("where", tOrP, q);
        }
        joinDateRange(start, end, step) {
          return this.append(["joinDateRange", "@0,@1,@2", start, end, step]);
        }
        selectWith(model, tOrP, q) {
          const nq = this.append(["selectWith", model.name]);
          return nq.process("select", tOrP, q);
        }
        select(tOrP, q) {
          return this.process("select", tOrP, q);
        }
        whereLinq(query, ...args) {
          let filters = "";
          const params = [];
          for (let index = 0; index < args.length; index++) {
            const element = args[index];
            const raw = query.raw[index];
            if (raw) {
              filters += raw;
            }
            const pi = `@${index}`;
            filters += pi;
            params.push(element);
          }
          const last = query.raw[args.length];
          if (last) {
            filters += last;
          }
          return this.append(["where", filters, ...params]);
        }
        selectLinq(query, ...args) {
          let filters = "";
          const params = [];
          for (let index = 0; index < args.length; index++) {
            const element = args[index];
            const raw = query.raw[index];
            if (raw) {
              filters += raw;
            }
            const pi = `@${index}`;
            filters += pi;
            params.push(element);
          }
          const last = query.raw[args.length];
          if (last) {
            filters += last;
          }
          return this.append(["select", filters, ...params]);
        }
        join(model, left, right) {
          return this.append(["join", model.name, left.toString(), right.toString()]);
        }
        include(tOrP, ...q) {
          if (typeof tOrP === "string") {
            return this.append(["include", tOrP], ...q.map(s => ["include", s]));
          }
          const select = convertToLinq(tOrP.toString());
          return this.append(["include", select]);
        }
        firstOrDefault(p = {}) {
          return __awaiter(this, void 0, void 0, function* () {
            const lp = p;
            lp.size = 1;
            lp.start = 0;
            lp.count = false;
            const list = yield this.toPagedList(lp);
            return list.items[0];
          });
        }
        orderBy(tOrP, q) {
          return this.process("orderBy", tOrP, q);
        }
        orderByDescending(tOrP, q) {
          return this.process("orderByDescending", tOrP, q);
        }
        thenBy(tOrP, q) {
          return this.process("thenBy", tOrP, q);
        }
        thenByDescending(tOrP, q) {
          return this.process("thenByDescending", tOrP, q);
        }
        trace() {
          this.context.traceQuery = true;
          return this;
        }
        toArray({
          cancelToken,
          doNotResolve,
          hideActivityIndicator,
          cacheSeconds,
          cacheVersion
        } = {}) {
          return __awaiter(this, void 0, void 0, function* () {
            const r = yield this.toPagedList({
              size: -1,
              count: false,
              cacheSeconds,
              cancelToken,
              doNotResolve,
              hideActivityIndicator,
              cacheVersion
            });
            return r.items;
          });
        }
        toPagedList({
          start = 0,
          size = 100,
          cancelToken,
          cacheVersion = "1",
          hideActivityIndicator,
          splitInclude = false,
          cacheSeconds = 0,
          count = true
        } = {}) {
          let url;
          const {
            traceQuery,
            name,
            service,
            queryFunction,
            args
          } = this.context;
          const trace = traceQuery;
          const methods = JSON.stringify(this.methods);
          const fm = new URLSearchParams();
          fm.append("methods", methods);
          if (start) {
            fm.append("start", start.toString());
          }
          if (size) {
            fm.append("size", size.toString());
          }
          fm.append("count", count.toString());
          if (trace) {
            fm.append("trace", trace ? "true" : "false");
          }
          if (queryFunction) {
            fm.append("function", queryFunction);
            fm.append("args", JSON.stringify(args !== null && args !== void 0 ? args : "[]"));
          }
          const encodedMethods = fm.toString();
          if (encodedMethods.length > 1824) {
            if (cacheSeconds > 0) {
              throw new Error("Generated query too big for caching");
            }
            url = `${service.url}methods/${name}`;
            return service.postJson({
              url,
              cancelToken,
              body: {
                methods: this.methods,
                start,
                size,
                split: splitInclude,
                count,
                trace,
                function: queryFunction || void 0,
                arguments: queryFunction ? args : void 0
              }
            });
          }
          if (cacheSeconds > 0) {
            fm.append("cache", cacheSeconds.toString());
            fm.append("cv", cacheVersion);
          } else {}
          url = `${service.url}query/${name}?${fm.toString()}`;
          return service.getJson({
            url,
            cancelToken,
            hideActivityIndicator
          });
        }
        thenInclude(a) {
          const select = convertToLinq(a.toString());
          return this.append(["thenInclude", select]);
        }
        append(...methods) {
          return new Query(this.context, [...this.methods, ...methods]);
        }
        process(name, tOrP, q) {
          if (!q) {
            const select = convertToLinq(tOrP.toString());
            return this.append([name, select]);
          }
          const pl = [];
          let i = 0;
          const p = tOrP;
          let text = q(p).toString();
          const pfn = StringHelper.findParameter(q.toString()).trim();
          for (const key in p) {
            if (Object.prototype.hasOwnProperty.call(p, key)) {
              const element = p[key];
              const pn = `@${i++}`;
              text = text.split(`${pfn}.${key}`).join(pn);
              pl.push(element);
            }
          }
          text = convertToLinq(text);
          return this.append([name, text, ...pl]);
        }
      });
    }
  };
});
//# sourceMappingURL=Query.js.map